from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *
from Products.Archetypes.exceptions import ReferenceException

from Products.Relations import exception, interfaces, jig
from Products.Relations.config import *

class CardinalityConstraint(BaseContent, jig.JigComponentBase):
    """An IValidator and IReferenceLayerProvider that enforces cardinality."""
    __implements__ = BaseContent.__implements__ + \
                     (interfaces.IValidator,
                      interfaces.IReferenceLayerProvider)

    content_icon = 'cardinalityconstraint_icon.gif'

    def validateConnected(self, reference, chain):
        self._validate(reference, chain)

    def validateDisconnected(self, reference, chain):
        self._validate(reference, chain)

    def _validate(self, reference, chain):
        rc = getToolByName(self, REFERENCE_CATALOG)
        search = rc(sourceUID=reference.sourceUID,
                    relationship=reference.relationship)
        targets = len(search)

        self.doValidate(targets, reference, chain)

    def doValidate(self, targets, reference, chain=None):
        jig = self.getJig()
        mintc = self.getMinTargetCardinality()
        maxtc = self.getMaxTargetCardinality()

        # XXX: i18n
        if mintc:
            if targets < mintc:
                raise exception.ValidationException(
                    "Too few targets (%s) for '%s'." % (targets, jig.Title()),
                    reference, chain)

        if maxtc:
            if targets > maxtc:
                raise exception.ValidationException(
                    "Too many targets (%s) for '%s'." % (targets, jig.Title()),
                    reference, chain)

    def provideReferenceLayer(self, reference): # IReferenceLayerProvider
        return CardinalityReferenceLayer(self.getJig(), self)
    
    # AT implementation    
    schema = BaseSchema + Schema((
        IntegerField('minTargetCardinality'),
        IntegerField('maxTargetCardinality'),
        ))
    portal_type = 'Cardinality Constraint'

registerType(CardinalityConstraint)


class CardinalityReferenceLayer:
    __implements__ = interfaces.IReferenceLayerProvider,

    def __init__(self, jig, cc):
        self.jig = jig
        self.component = cc # cardinality constraint

    def beforeTargetDeleteInformSource(self, reference):
        rc = getToolByName(self.jig, REFERENCE_CATALOG)
        search = rc(sourceUID=reference.sourceUID,
                    relationship=self.jig.getId())
        targets = len(search) - 1

        try:
            self.component.doValidate(targets, reference)
        except exception.ValidationException, e:
            raise ReferenceException, e
    
