from Acquisition import ImplicitAcquisitionWrapper, aq_parent
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *
from Products.Archetypes.ReferenceEngine import Reference
from Products.Archetypes.utils import make_uuid

from Products.Relations.config import *
from Products.Relations import interfaces, jig, utils

class InverseImplicator(BaseContent, jig.JigComponentBase):
    """Implicator that creates a reference from target to source."""
    __implements__ = (interfaces.IImplicator,) + BaseContent.__implements__

    content_icon = 'inverseimplicator_icon.gif'

    def implyOnConnect(self, reference, chain):
        ijig = self.getInverseJig()
        if ijig is not None:
            ijig.implyOnConnect(reference.getTargetBrain(),
                                reference.getSourceBrain(),
                                chain)

    def implyOnDisconnect(self, reference, chain):
        ijig = self.getInverseJig()
        if ijig is None:
            return

        rc = getToolByName(self.getJig(), REFERENCE_CATALOG)
        result = rc(sourceUID=reference.targetUID,
                    targetUID=reference.sourceUID,
                    relationship=ijig.getId())
        if result:
            ijig.implyOnDisconnect(result[0].getObject(), chain)

    # Archetypes implementation
    schema = BaseSchema + Schema((
        ReferenceField('inverseJig',
                       relationship='inverseJig',
                       allowed_types_method='_allowedTypesJig'),
        ))
    portal_type = 'Inverse Implicator'

    def _allowedTypesJig(self, instance):
        return utils.getPortalTypesByInterfaces(self, ('IReferenceJig',))

    def getInverseJig(self):
        inv = self.getRefs('inverseJig')
        if inv:
            return inv[0]
        else:
            return None

registerType(InverseImplicator)
                       
        

    
