from AccessControl import ModuleSecurityInfo
from Products.CMFCore.utils import getToolByName

from config import *
import brain
import interfaces

modulesec = ModuleSecurityInfo('Products.Relations.processor')
modulesec.declarePublic('process')

class Chain(dict):
    __implements__ = interfaces.IChain,
    
    def __init__(self):
        self.added = []
        self.deleted = []

    def __getitem__(self, key):
        if not key in self.keys():
            self[key] = {}
        return dict.__getitem__(self, key)



# module implements the interface
__implements__ = interfaces.IReferenceConnectionProcessor,

def process(context, connect=(), disconnect=()):
    get_transaction().commit(1)
    try:
        return _process(context, connect, disconnect)
    except:
        get_transaction().abort(1)
        raise

def _process(context, connect, disconnect):
    registry = getToolByName(context, REFERENCEJIGREGISTRY)
    ref_ctl = getToolByName(context, REFERENCE_CATALOG)

    chain = Chain()
    seen_jigs = {} # avoid many lookups

    # implyOnConnect
    for sUID, tUID, relationship in connect:
        jig = _lookup(relationship, registry, seen_jigs)
        sbrain = brain.makeBrainAggregate(context, sUID)
        tbrain = brain.makeBrainAggregate(context, tUID)
        jig.implyOnConnect(sbrain, tbrain, chain)

    # implyOnDisconnect
    for sUID, tUID, relationship in disconnect:
        jig = _lookup(relationship, registry, seen_jigs)
        refs = ref_ctl(sourceUID=sUID, targetUID=tUID,
                       relationship=relationship)
        if refs:
            jig.implyOnDisconnect(refs[0].getObject(), chain)

    # validate and validateDeletion
    _validate(context, chain, seen_jigs)
    
    # finalizeOnConnect and finalizeOnDisconnect
    _finalize(context, chain, seen_jigs)

    return chain

def _validate(context, chain, seen_jigs):
    registry = getToolByName(context, REFERENCEJIGREGISTRY)

    for reference in chain.added:
        jig = _lookup(reference.relationship, registry, seen_jigs)
        jig.validateConnected(reference, chain)

    for reference in chain.deleted:
        jig = _lookup(reference.relationship, registry, seen_jigs)
        jig.validateDisconnected(reference, chain)

def _finalize(context, chain, seen_jigs):
    registry = getToolByName(context, REFERENCEJIGREGISTRY)
    
    for reference in chain.added:
        jig = _lookup(reference.relationship, registry, seen_jigs)
        jig.finalizeOnConnect(reference, chain)
        
    for reference in chain.deleted:
        jig = _lookup(reference.relationship, registry, seen_jigs)
        jig.finalizeOnDisconnect(reference, chain)
        
def _lookup(relationship, registry, seen_jigs):
    if not seen_jigs.has_key(relationship):
        seen_jigs[relationship] = registry.getJig(relationship)
    return seen_jigs[relationship]
